 MODULE mod_chem_spack_kinetic
  
   IMPLICIT NONE
   PRIVATE
   PUBLIC :: kinetic ! subroutine
 CONTAINS
   SUBROUTINE kinetic(jppj,Jphoto,rk,temp,xlw,Press,cosz,att,ijkbeg,ijkend,maxblock_size,nr)
 
!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!
!     This routine computes the kinetic rates for the gas-phase.
!     This routine is automatically generated by SPACK.
!     Mechanism: ../Mechanism/CB07   
!     Species: ../Mechanism/ciCB07 
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!
!     TEMP: temperature ([K]).
!     XLW: water massic fraction.
!     PRESS: pressure ([Pa]).
!     AZI: zenithal angle ([degree]).
!     ATT: attenuation variable.
!
!     -- INPUT/OUTPUT VARIABLES
!
!     -- OUTPUT VARIABLES
!
!     RK: kinetic rates.
!
!------------------------------------------------------------------------
!
!     -- REMARKS
!
!------------------------------------------------------------------------
!
!     -- MODIFICATIONS
!
!------------------------------------------------------------------------
!
!     -- AUTHOR(S)
!
!     SPACK.
!
!------------------------------------------------------------------------
 
      IMPLICIT NONE
 
 
 
      INTEGER,INTENT(IN) :: jppj,ijkbeg,ijkend,maxblock_size,nr
      DOUBLE PRECISION,INTENT(IN) ::  xlw(maxblock_size),att(maxblock_size),cosz(maxblock_size)
      DOUBLE PRECISION,INTENT(IN) ::  temp(maxblock_size),Press(maxblock_size)
      DOUBLE PRECISION,INTENT(OUT) ::  rk(maxblock_size,nr)
      DOUBLE PRECISION,DIMENSION(maxblock_size) ::  Effko,Rapk,facteur
      DOUBLE PRECISION,DIMENSION(maxblock_size) :: YlH2O,SumM,azi
      DOUBLE PRECISION,INTENT(IN) ::  Jphoto(maxblock_size,jppj)
      INTEGER :: ijk
 
!     Compute third body.
!     Conversion = Avogadro*1d-6/Perfect gas constant.
!     PRESS in Pascal, SUMM in molecules/cm3, TEMP in Kelvin
 
      DO ijk=ijkbeg,ijkend
        SumM(ijk) = Press(ijk) * 7.243D16 / temp(ijk)
      END DO
 
!     Number of water molecules computed from the massic fraction
!     (absolute humidity)
 
      DO ijk=ijkbeg,ijkend
         YlH2O(ijk) = 29.d0*SumM(ijk)*xlw(ijk)/(18.d0+11.d0*xlw(ijk))
      END DO
 
!     For the zenithal angle at tropics
 
      azi=abs(acos(cosz)*180.0/3.1415926) !From cosz obtain angle azi in degrees
 
 
       DO ijk=ijkbeg,ijkend
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  1)=-0.1302720567168789D-07
       rk(ijk,  1)=-0.7822279432831316D-06+(azi(ijk)- 0.00D+00) * rk(ijk,  1)
       rk(ijk,  1)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  1)
       rk(ijk,  1)= 0.9310260000000001D-02+(azi(ijk)- 0.00D+00) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  1)= 0.3771617015066953D-08
       rk(ijk,  1)=-0.1173044113433769D-05+(azi(ijk)- 0.10D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.1955272056716900D-04+(azi(ijk)- 0.10D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.9219010000000000D-02+(azi(ijk)- 0.10D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  1)=-0.5859262388581603D-08
       rk(ijk,  1)=-0.1059895602981760D-05+(azi(ijk)- 0.20D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.4188211773132429D-04+(azi(ijk)- 0.20D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.8909950000000000D-02+(azi(ijk)- 0.20D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  1)=-0.7024567460739109D-08
       rk(ijk,  1)=-0.1235673474639209D-05+(azi(ijk)- 0.30D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.6483780850753396D-04+(azi(ijk)- 0.30D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.8379279999999999D-02+(azi(ijk)- 0.30D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  1)=-0.9202467768463278D-08
       rk(ijk,  1)=-0.1446410498461379D-05+(azi(ijk)- 0.40D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.9165864823853988D-04+(azi(ijk)- 0.40D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.7600310000000000D-02+(azi(ijk)- 0.40D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  1)=-0.1612556146541203D-07
       rk(ijk,  1)=-0.1722484531515246D-05+(azi(ijk)- 0.50D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.1233475985383064D-03+(azi(ijk)- 0.50D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.6529880000000000D-02+(azi(ijk)- 0.50D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  1)= 0.3226471363012407D-07
       rk(ijk,  1)=-0.2206251375477739D-05+(azi(ijk)- 0.60D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.1626349576082347D-03+(azi(ijk)- 0.60D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.5108030000000000D-02+(azi(ijk)- 0.60D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  1)= 0.2027078243960391D-06
       rk(ijk,  1)=-0.1238309966574421D-05+(azi(ijk)- 0.70D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.1970805710287534D-03+(azi(ijk)- 0.70D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.3293320000000000D-02+(azi(ijk)- 0.70D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  1)=-0.7448311471175039D-07
       rk(ijk,  1)= 0.3626677818932096D-05+(azi(ijk)- 0.78D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.1779736282099075D-03+(azi(ijk)- 0.78D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.1741210000000000D-02+(azi(ijk)- 0.78D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  1)= 0.2490309929267239D-05
       rk(ijk,  1)= 0.1839083065847241D-05+(azi(ijk)- 0.86D+02) * rk(ijk,  1)
       rk(ijk,  1)=-0.1342475411316550D-03+(azi(ijk)- 0.86D+02) * rk(ijk,  1)
       rk(ijk,  1)= 0.5113930000000000D-03+(azi(ijk)- 0.86D+02) * rk(ijk,  1)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  1)= 0.1632080000000000D-03
      endif
      rk(ijk,  1) = rk(ijk,  1) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  2)=-0.8776629099833420D-10
       rk(ijk,  2)=-0.1165033709001662D-07+(azi(ijk)- 0.00D+00) * rk(ijk,  2)
       rk(ijk,  2)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  2)
       rk(ijk,  2)= 0.3523480000000000D-04+(azi(ijk)- 0.00D+00) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  2)= 0.1474988729949891D-09
       rk(ijk,  2)=-0.1428332581996664D-07+(azi(ijk)- 0.10D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.2593366290998326D-06+(azi(ijk)- 0.10D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.3398200000000000D-04+(azi(ijk)- 0.10D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  2)= 0.1300707990183873D-09
       rk(ijk,  2)=-0.9858359630116980D-08+(azi(ijk)- 0.20D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.5007534836006687D-06+(azi(ijk)- 0.20D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.3010780000000000D-04+(azi(ijk)- 0.20D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  2)= 0.1988179309314557D-09
       rk(ijk,  2)=-0.5956235659565352D-08+(azi(ijk)- 0.30D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.6588994364974921D-06+(azi(ijk)- 0.30D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.2424450000000000D-04+(azi(ijk)- 0.30D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  2)= 0.2219574772557704D-09
       rk(ijk,  2)= 0.8302268378494226D-11+(azi(ijk)- 0.40D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.7183787704093628D-06+(azi(ijk)- 0.40D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.1725870000000000D-04+(azi(ijk)- 0.40D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  2)= 0.1913521600454229D-09
       rk(ijk,  2)= 0.6667026586051454D-08+(azi(ijk)- 0.50D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.6516254818650610D-06+(azi(ijk)- 0.50D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.1029770000000000D-04+(azi(ijk)- 0.50D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  2)= 0.1602388256259833D-10
       rk(ijk,  2)= 0.1240759138741341D-07+(azi(ijk)- 0.60D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.4608793021304060D-06+(azi(ijk)- 0.60D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.4639500000000000D-05+(azi(ijk)- 0.60D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  2)=-0.3089359890776449D-09
       rk(ijk,  2)= 0.1288830786429022D-07+(azi(ijk)- 0.70D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.2079203096133862D-06+(azi(ijk)- 0.70D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.1287490000000000D-05+(azi(ijk)- 0.70D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  2)=-0.2034952628702240D-09
       rk(ijk,  2)= 0.5473844126423800D-08+(azi(ijk)- 0.78D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.6102309368774707D-07+(azi(ijk)- 0.78D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.2908040000000000D-06+(azi(ijk)- 0.78D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  2)= 0.1623544915691508D-09
       rk(ijk,  2)= 0.5899578174960450D-09+(azi(ijk)- 0.86D+02) * rk(ijk,  2)
       rk(ijk,  2)=-0.1251267813614395D-07+(azi(ijk)- 0.86D+02) * rk(ijk,  2)
       rk(ijk,  2)= 0.4875570000000000D-07+(azi(ijk)- 0.86D+02) * rk(ijk,  2)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  2)= 0.1853500000000000D-07
      endif
      rk(ijk,  2) = rk(ijk,  2) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  3)=-0.5928286648808007D-09
       rk(ijk,  3)=-0.3096171335119278D-07+(azi(ijk)- 0.00D+00) * rk(ijk,  3)
       rk(ijk,  3)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  3)
       rk(ijk,  3)= 0.4927580000000000D-03+(azi(ijk)- 0.00D+00) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  3)= 0.1444859946426951D-09
       rk(ijk,  3)=-0.4874657329761681D-07+(azi(ijk)- 0.10D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.7970828664880959D-06+(azi(ijk)- 0.10D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.4890690000000000D-03+(azi(ijk)- 0.10D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  3)=-0.5511531369015992D-10
       rk(ijk,  3)=-0.4441199345833597D-07+(azi(ijk)- 0.20D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.1728668534047624D-05+(azi(ijk)- 0.20D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.4763680000000000D-03+(azi(ijk)- 0.20D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  3)=-0.3000247398820652D-09
       rk(ijk,  3)=-0.4606545286904076D-07+(azi(ijk)- 0.30D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.2633442997321390D-05+(azi(ijk)- 0.30D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.4545850000000000D-03+(azi(ijk)- 0.30D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  3)=-0.2397857267814825D-09
       rk(ijk,  3)=-0.5506619506550265D-07+(azi(ijk)- 0.40D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.3644759476666825D-05+(azi(ijk)- 0.40D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.4233440000000000D-03+(azi(ijk)- 0.40D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  3)=-0.1844832352992110D-08
       rk(ijk,  3)=-0.6225976686894679D-07+(azi(ijk)- 0.50D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.4818019096011324D-05+(azi(ijk)- 0.50D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.3811500000000000D-03+(azi(ijk)- 0.50D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  3)= 0.1101151387498967D-09
       rk(ijk,  3)=-0.1176047374587144D-06+(azi(ijk)- 0.60D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.6616664139287888D-05+(azi(ijk)- 0.60D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.3248990000000000D-03+(azi(ijk)- 0.60D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  3)=-0.1557211541864023D-07
       rk(ijk,  3)=-0.1143012832962279D-06+(azi(ijk)- 0.70D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.8935724346837256D-05+(azi(ijk)- 0.70D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.2470820000000000D-03+(azi(ijk)- 0.70D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  3)= 0.4048472604225035D-07
       rk(ijk,  3)=-0.4880320533435654D-06+(azi(ijk)- 0.78D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.1375439103995572D-04+(azi(ijk)- 0.78D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.1603080000000000D-03+(azi(ijk)- 0.78D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  3)= 0.2066880316661520D-06
       rk(ijk,  3)= 0.4836013716701137D-06+(azi(ijk)- 0.86D+02) * rk(ijk,  3)
       rk(ijk,  3)=-0.1378983649333980D-04+(azi(ijk)- 0.86D+02) * rk(ijk,  3)
       rk(ijk,  3)= 0.3976700000000000D-04+(azi(ijk)- 0.86D+02) * rk(ijk,  3)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  3)= 0.5573310000000000D-05
      endif
      rk(ijk,  3) = rk(ijk,  3) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  4)=-0.2887225450832654D-08
       rk(ijk,  4)=-0.1810277454916759D-06+(azi(ijk)- 0.00D+00) * rk(ijk,  4)
       rk(ijk,  4)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  4)
       rk(ijk,  4)= 0.2046710000000000D-02+(azi(ijk)- 0.00D+00) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  4)= 0.8216763524985514D-09
       rk(ijk,  4)=-0.2676445090166556D-06+(azi(ijk)- 0.10D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.4486722545083314D-05+(azi(ijk)- 0.10D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.2025720000000000D-02+(azi(ijk)- 0.10D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  4)=-0.1309479959161345D-08
       rk(ijk,  4)=-0.2429942184416991D-06+(azi(ijk)- 0.20D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.9593109819666860D-05+(azi(ijk)- 0.20D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.1954910000000000D-02+(azi(ijk)- 0.20D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  4)=-0.1523756515853734D-08
       rk(ijk,  4)=-0.2822786172165392D-06+(azi(ijk)- 0.30D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.1484583817624924D-04+(azi(ijk)- 0.30D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.1833370000000000D-02+(azi(ijk)- 0.30D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  4)=-0.1745493977423840D-08
       rk(ijk,  4)=-0.3279913126921493D-06+(azi(ijk)- 0.40D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.2094853747533615D-04+(azi(ijk)- 0.40D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.1655160000000000D-02+(azi(ijk)- 0.40D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  4)=-0.2754267574452238D-08
       rk(ijk,  4)=-0.3803561320148585D-06+(azi(ijk)- 0.50D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.2803201192240627D-04+(azi(ijk)- 0.50D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.1411130000000000D-02+(azi(ijk)- 0.50D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  4)= 0.1037656427522960D-07
       rk(ijk,  4)=-0.4629841592484437D-06+(azi(ijk)- 0.60D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.3646541483503906D-04+(azi(ijk)- 0.60D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.1090020000000000D-02+(azi(ijk)- 0.60D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  4)= 0.4335158727144718D-07
       rk(ijk,  4)=-0.1516872309916274D-06+(azi(ijk)- 0.70D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.4261212873743960D-04+(azi(ijk)- 0.70D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.6894440000000000D-03+(azi(ijk)- 0.70D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  4)=-0.2976581610935105D-07
       rk(ijk,  4)= 0.8887508635231610D-06+(azi(ijk)- 0.78D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.3671561967718845D-04+(azi(ijk)- 0.78D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.3610350000000000D-03+(azi(ijk)- 0.78D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  4)= 0.5586598403877321D-06
       rk(ijk,  4)= 0.1743712768980777D-06+(azi(ijk)- 0.86D+02) * rk(ijk,  4)
       rk(ijk,  4)=-0.2821064255380800D-04+(azi(ijk)- 0.86D+02) * rk(ijk,  4)
       rk(ijk,  4)= 0.1089500000000000D-03+(azi(ijk)- 0.86D+02) * rk(ijk,  4)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  4)= 0.3465160000000000D-04
      endif
      rk(ijk,  4) = rk(ijk,  4) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  5)=-0.1532025548735493D-11
       rk(ijk,  5)=-0.1605297445126457D-09+(azi(ijk)- 0.00D+00) * rk(ijk,  5)
       rk(ijk,  5)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  5)
       rk(ijk,  5)= 0.7475310000000001D-06+(azi(ijk)- 0.00D+00) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  5)= 0.1499076646206643D-11
       rk(ijk,  5)=-0.2064905109747105D-09+(azi(ijk)- 0.10D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.3670202554873561D-08+(azi(ijk)- 0.10D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.7299460000000000D-06+(azi(ijk)- 0.10D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  5)= 0.8827189639089452D-12
       rk(ijk,  5)=-0.1615182115885113D-09+(azi(ijk)- 0.20D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.7350289780505779D-08+(azi(ijk)- 0.20D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.6740940000000000D-06+(azi(ijk)- 0.20D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  5)= 0.1742047498157467D-11
       rk(ijk,  5)=-0.1350366426712434D-09+(azi(ijk)- 0.30D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.1031583832310332D-07+(azi(ijk)- 0.30D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.5853220000000000D-06+(azi(ijk)- 0.30D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  5)= 0.2513091043461019D-11
       rk(ijk,  5)=-0.8277521772651680D-10+(azi(ijk)- 0.40D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.1249395692708095D-07+(azi(ijk)- 0.40D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.4704020000000000D-06+(azi(ijk)- 0.40D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  5)= 0.3194588327999034D-11
       rk(ijk,  5)=-0.7382486422686429D-11+(azi(ijk)- 0.50D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.1339553396857297D-07+(azi(ijk)- 0.50D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.3396980000000000D-06+(azi(ijk)- 0.50D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  5)= 0.5281855644544062D-11
       rk(ijk,  5)= 0.8845516341725539D-10+(azi(ijk)- 0.60D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.1258480719862693D-07+(azi(ijk)- 0.60D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.2081990000000000D-06+(azi(ijk)- 0.60D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  5)=-0.2756160173518845D-12
       rk(ijk,  5)= 0.2469108327535101D-09+(azi(ijk)- 0.70D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.9231147236918914D-08+(azi(ijk)- 0.70D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.9647830000000000D-07+(azi(ijk)- 0.70D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  5)=-0.5270116382857717D-11
       rk(ijk,  5)= 0.2402960483371050D-09+(azi(ijk)- 0.78D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.5333492188196593D-08+(azi(ijk)- 0.78D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.3829030000000000D-07+(azi(ijk)- 0.78D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  5)= 0.3312732748143745D-10
       rk(ijk,  5)= 0.1138132551483600D-09+(azi(ijk)- 0.86D+02) * rk(ijk,  5)
       rk(ijk,  5)=-0.2500617760306919D-08+(azi(ijk)- 0.86D+02) * rk(ijk,  5)
       rk(ijk,  5)= 0.8303010000000000D-08+(azi(ijk)- 0.86D+02) * rk(ijk,  5)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  5)= 0.2241700000000000D-08
      endif
      rk(ijk,  5) = rk(ijk,  5) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  6)=-0.1022637521258295D-10
       rk(ijk,  6)=-0.1047636247874169D-08+(azi(ijk)- 0.00D+00) * rk(ijk,  6)
       rk(ijk,  6)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  6)
       rk(ijk,  6)= 0.5038360000000000D-05+(azi(ijk)- 0.00D+00) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  6)= 0.9289125637748354D-11
       rk(ijk,  6)=-0.1354427504251658D-08+(azi(ijk)- 0.10D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.2402063752125827D-07+(azi(ijk)- 0.10D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.4923370000000000D-05+(azi(ijk)- 0.10D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  6)= 0.4979872661590801D-11
       rk(ijk,  6)=-0.1075753735119208D-08+(azi(ijk)- 0.20D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.4832244991496693D-07+(azi(ijk)- 0.20D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.4557010000000000D-05+(azi(ijk)- 0.20D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  6)= 0.1055138371588663D-10
       rk(ijk,  6)=-0.9263575552714857D-09+(azi(ijk)- 0.30D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.6834356281887383D-07+(azi(ijk)- 0.30D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.3971190000000000D-05+(azi(ijk)- 0.30D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  6)= 0.1620459247486142D-10
       rk(ijk,  6)=-0.6098160437948826D-09+(azi(ijk)- 0.40D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.8370529880953758D-07+(azi(ijk)- 0.40D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.3205670000000000D-05+(azi(ijk)- 0.40D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  6)= 0.2215024638466714D-10
       rk(ijk,  6)=-0.1236782695490249D-09+(azi(ijk)- 0.50D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.9104024194297691D-07+(azi(ijk)- 0.50D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.2323840000000000D-05+(azi(ijk)- 0.50D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  6)= 0.3999042198646269D-10
       rk(ijk,  6)= 0.5408291219909645D-09+(azi(ijk)- 0.60D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.8686873341855749D-07+(azi(ijk)- 0.60D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.1423220000000000D-05+(azi(ijk)- 0.60D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  6)=-0.2067341716793420D-11
       rk(ijk,  6)= 0.1740541781584667D-08+(azi(ijk)- 0.70D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.6405502438280023D-07+(azi(ijk)- 0.70D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.6486060000000000D-06+(azi(ijk)- 0.70D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  6)=-0.2916729149970486D-10
       rk(ijk,  6)= 0.1690925580381730D-08+(azi(ijk)- 0.78D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.3660328548706694D-07+(azi(ijk)- 0.78D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.2465020000000000D-06+(azi(ijk)- 0.78D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  6)= 0.1504439894566837D-09
       rk(ijk,  6)= 0.9909105843816125D-09+(azi(ijk)- 0.86D+02) * rk(ijk,  6)
       rk(ijk,  6)=-0.1514859616887210D-07+(azi(ijk)- 0.86D+02) * rk(ijk,  6)
       rk(ijk,  6)= 0.4696130000000000D-07+(azi(ijk)- 0.86D+02) * rk(ijk,  6)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  6)= 0.1184990000000000D-07
      endif
      rk(ijk,  6) = rk(ijk,  6) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  7)=-0.3691161034363460D-07
       rk(ijk,  7)=-0.1177883896563626D-05+(azi(ijk)- 0.00D+00) * rk(ijk,  7)
       rk(ijk,  7)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  7)
       rk(ijk,  7)= 0.2656490000000000D-01+(azi(ijk)- 0.00D+00) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  7)= 0.1703483103089474D-07
       rk(ijk,  7)=-0.2285232206872664D-05+(azi(ijk)- 0.10D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.3463116103436289D-04+(azi(ijk)- 0.10D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.2641020000000000D-01+(azi(ijk)- 0.10D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  7)=-0.1532771377994151D-07
       rk(ijk,  7)=-0.1774187275945823D-05+(azi(ijk)- 0.20D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.7522535586254776D-04+(azi(ijk)- 0.20D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.2585240000000000D-01+(azi(ijk)- 0.20D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  7)=-0.1372397591112777D-07
       rk(ijk,  7)=-0.2234018689344070D-05+(azi(ijk)- 0.30D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.1153074155154467D-03+(azi(ijk)- 0.30D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.2490740000000000D-01+(azi(ijk)- 0.30D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  7)=-0.2747638257554609D-07
       rk(ijk,  7)=-0.2645737966677890D-05+(azi(ijk)- 0.40D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.1641049820756664D-03+(azi(ijk)- 0.40D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.2351720000000000D-01+(azi(ijk)- 0.40D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  7)=-0.1322704937866908D-06
       rk(ijk,  7)=-0.3470029443944269D-05+(azi(ijk)- 0.50D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.2252626561818881D-03+(azi(ijk)- 0.50D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.2158410000000000D-01+(azi(ijk)- 0.50D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  7)=-0.2304164227770285D-07
       rk(ijk,  7)=-0.7438144257545083D-05+(azi(ijk)- 0.60D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.3343443931967814D-03+(azi(ijk)- 0.60D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.1885220000000000D-01+(azi(ijk)- 0.60D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  7)=-0.1151055949406269D-05
       rk(ijk,  7)=-0.8129393525876153D-05+(azi(ijk)- 0.70D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.4900197710309918D-03+(azi(ijk)- 0.70D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.1474190000000000D-01+(azi(ijk)- 0.70D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  7)= 0.3051007034750409D-05
       rk(ijk,  7)=-0.3575473631162467D-04+(azi(ijk)- 0.78D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.8410928097310252D-03+(azi(ijk)- 0.78D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.9712119999999999D-02+(azi(ijk)- 0.78D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  7)= 0.1099207874720812D-04
       rk(ijk,  7)= 0.3746943252237739D-04+(azi(ijk)- 0.86D+02) * rk(ijk,  7)
       rk(ijk,  7)=-0.8273752400449071D-03+(azi(ijk)- 0.86D+02) * rk(ijk,  7)
       rk(ijk,  7)= 0.2257190000000000D-02+(azi(ijk)- 0.86D+02) * rk(ijk,  7)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  7)= 0.2506930000000000D-03
      endif
      rk(ijk,  7) = rk(ijk,  7) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  8)=-0.1926445199576596D-06
       rk(ijk,  8)=-0.1114355480042343D-04+(azi(ijk)- 0.00D+00) * rk(ijk,  8)
       rk(ijk,  8)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  8)
       rk(ijk,  8)= 0.2129960000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  8)=-0.3806644012699886D-07
       rk(ijk,  8)=-0.1692289039915322D-04+(azi(ijk)- 0.10D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.2806644519957665D-03+(azi(ijk)- 0.10D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.2116890000000000D+00+(azi(ijk)- 0.10D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  8)= 0.3591028046562472D-07
       rk(ijk,  8)=-0.1806488360296319D-04+(azi(ijk)- 0.20D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.6305421920169306D-03+(azi(ijk)- 0.20D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.2071520000000000D+00+(azi(ijk)- 0.20D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  8)=-0.2185746817355059D-06
       rk(ijk,  8)=-0.1698757518899445D-04+(azi(ijk)- 0.30D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.9810667799365067D-03+(azi(ijk)- 0.30D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.1990760000000000D+00+(azi(ijk)- 0.30D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  8)=-0.2006115535235708D-06
       rk(ijk,  8)=-0.2354481564105945D-04+(azi(ijk)- 0.40D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.1386390688237047D-02+(azi(ijk)- 0.40D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.1873480000000000D+00+(azi(ijk)- 0.40D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  8)=-0.1055979104170286D-05
       rk(ijk,  8)=-0.2956316224676663D-04+(azi(ijk)- 0.50D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.1917470467115308D-02+(azi(ijk)- 0.50D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.1709290000000000D+00+(azi(ijk)- 0.50D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  8)=-0.3204720297952648D-06
       rk(ijk,  8)=-0.6124253537187633D-04+(azi(ijk)- 0.60D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.2825527443301725D-02+(azi(ijk)- 0.60D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.1477420000000000D+00+(azi(ijk)- 0.60D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  8)=-0.6785572971816661D-05
       rk(ijk,  8)=-0.7085669626573617D-04+(azi(ijk)- 0.70D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.4146519759677833D-02+(azi(ijk)- 0.70D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.1130420000000000D+00+(azi(ijk)- 0.70D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  8)= 0.2349184361302091D-04
       rk(ijk,  8)=-0.2337104475893259D-03+(azi(ijk)- 0.78D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.6583056910518562D-02+(azi(ijk)- 0.78D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.7186080000000000D-01+(azi(ijk)- 0.78D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  8)= 0.6606749385961857D-04
       rk(ijk,  8)= 0.3300937991231632D-03+(azi(ijk)- 0.86D+02) * rk(ijk,  8)
       rk(ijk,  8)=-0.5811990098247437D-02+(azi(ijk)- 0.86D+02) * rk(ijk,  8)
       rk(ijk,  8)= 0.1626670000000000D-01+(azi(ijk)- 0.86D+02) * rk(ijk,  8)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  8)= 0.2528560000000000D-02
      endif
      rk(ijk,  8) = rk(ijk,  8) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk,  9)=-0.1441479345432036D-10
       rk(ijk,  9)=-0.1242452065456794D-08+(azi(ijk)- 0.00D+00) * rk(ijk,  9)
       rk(ijk,  9)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk,  9)
       rk(ijk,  9)= 0.8394580000000000D-05+(azi(ijk)- 0.00D+00) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk,  9)= 0.8244380362960527D-11
       rk(ijk,  9)=-0.1674895869086406D-08+(azi(ijk)- 0.10D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.2917347934543199D-07+(azi(ijk)- 0.10D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.8255920000000000D-05+(azi(ijk)- 0.10D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk,  9)= 0.1172720024782291D-12
       rk(ijk,  9)=-0.1427564458197590D-08+(azi(ijk)- 0.20D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.6019808261827194D-07+(azi(ijk)- 0.20D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.7804940000000000D-05+(azi(ijk)- 0.20D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk,  9)= 0.4506531627126779D-11
       rk(ijk,  9)=-0.1424046298123243D-08+(azi(ijk)- 0.30D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.8871419018148029D-07+(azi(ijk)- 0.30D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.7060320000000000D-05+(azi(ijk)- 0.30D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk,  9)= 0.1086660148901067D-10
       rk(ijk,  9)=-0.1288850349309422D-08+(azi(ijk)- 0.40D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.1158431566558071D-06+(azi(ijk)- 0.40D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.6035280000000000D-05+(azi(ijk)- 0.40D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk,  9)= 0.1803706241682997D-10
       rk(ijk,  9)=-0.9628523046390844D-09+(azi(ijk)- 0.50D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.1383601831952922D-06+(azi(ijk)- 0.50D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.4758830000000000D-05+(azi(ijk)- 0.50D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk,  9)= 0.7329514884366502D-10
       rk(ijk,  9)=-0.4217404321342295D-09+(azi(ijk)- 0.60D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.1522061105630245D-06+(azi(ijk)- 0.60D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.3296980000000000D-05+(azi(ijk)- 0.60D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk,  9)= 0.6172011386261483D-10
       rk(ijk,  9)= 0.1777114033175403D-08+(azi(ijk)- 0.70D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.1386523745526125D-06+(azi(ijk)- 0.70D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.1806040000000000D-05+(azi(ijk)- 0.70D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk,  9)=-0.8279704635718142D-10
       rk(ijk,  9)= 0.3258396765879289D-08+(azi(ijk)- 0.78D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.9836828816018174D-07+(azi(ijk)- 0.78D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.8421570000000000D-06+(azi(ijk)- 0.78D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk,  9)= 0.1082517324585549D-08
       rk(ijk,  9)= 0.1271267653299030D-08+(azi(ijk)- 0.86D+02) * rk(ijk,  9)
       rk(ijk,  9)=-0.6213097280668286D-07+(azi(ijk)- 0.86D+02) * rk(ijk,  9)
       rk(ijk,  9)= 0.2213560000000000D-06+(azi(ijk)- 0.86D+02) * rk(ijk,  9)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk,  9)= 0.6245350000000000D-07
      endif
      rk(ijk,  9) = rk(ijk,  9) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk, 10)=-0.7261045436751569D-10
       rk(ijk, 10)=-0.5576895456324840D-08+(azi(ijk)- 0.00D+00) * rk(ijk, 10)
       rk(ijk, 10)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk, 10)
       rk(ijk, 10)= 0.4669460000000000D-04+(azi(ijk)- 0.00D+00) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk, 10)= 0.2853136310254520D-10
       rk(ijk, 10)=-0.7755209087350312D-08+(azi(ijk)- 0.10D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.1333210454367515D-06+(azi(ijk)- 0.10D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.4606430000000000D-04+(azi(ijk)- 0.10D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk, 10)=-0.1901499804266231D-10
       rk(ijk, 10)=-0.6899268194273959D-08+(azi(ijk)- 0.20D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.2798658182529942D-06+(azi(ijk)- 0.20D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.4398410000000000D-04+(azi(ijk)- 0.20D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk, 10)=-0.9471370931894209D-11
       rk(ijk, 10)=-0.7469718135553829D-08+(azi(ijk)- 0.30D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.4235556815512720D-06+(azi(ijk)- 0.30D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.4047650000000000D-04+(azi(ijk)- 0.30D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk, 10)= 0.1440048177021868D-10
       rk(ijk, 10)=-0.7753859263510583D-08+(azi(ijk)- 0.40D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.5757914555419167D-06+(azi(ijk)- 0.40D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.3548450000000000D-04+(azi(ijk)- 0.40D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk, 10)= 0.3856944385102582D-10
       rk(ijk, 10)=-0.7321844810403943D-08+(azi(ijk)- 0.50D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.7265484962810631D-06+(azi(ijk)- 0.50D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.2896560000000000D-04+(azi(ijk)- 0.50D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk, 10)= 0.4136217428256926D-09
       rk(ijk, 10)=-0.6164761494873299D-08+(azi(ijk)- 0.60D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.8614145593338335D-06+(azi(ijk)- 0.60D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.2100650000000000D-04+(azi(ijk)- 0.60D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk, 10)= 0.5376701572565394D-09
       rk(ijk, 10)= 0.6243890789895679D-08+(azi(ijk)- 0.70D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.8606232663835842D-06+(azi(ijk)- 0.70D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.1218950000000000D-04+(azi(ijk)- 0.70D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk, 10)=-0.5508273900085886D-09
       rk(ijk, 10)= 0.1914797456405770D-07+(azi(ijk)- 0.78D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.6574883435519911D-06+(azi(ijk)- 0.78D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.5979410000000000D-05+(azi(ijk)- 0.78D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk, 10)= 0.8530305662040679D-08
       rk(ijk, 10)= 0.5928117203844831D-08+(azi(ijk)- 0.86D+02) * rk(ijk, 10)
       rk(ijk, 10)=-0.4568796094084461D-06+(azi(ijk)- 0.86D+02) * rk(ijk, 10)
       rk(ijk, 10)= 0.1662950000000000D-05+(azi(ijk)- 0.86D+02) * rk(ijk, 10)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk, 10)= 0.4762210000000000D-06
      endif
      rk(ijk, 10) = rk(ijk, 10) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk, 11)=-0.5816837387219485D-10
       rk(ijk, 11)=-0.5184316261278090D-08+(azi(ijk)- 0.00D+00) * rk(ijk, 11)
       rk(ijk, 11)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk, 11)
       rk(ijk, 11)= 0.3220560000000000D-04+(azi(ijk)- 0.00D+00) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk, 11)= 0.3450512161659926D-10
       rk(ijk, 11)=-0.6929367477443936D-08+(azi(ijk)- 0.10D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.1211368373872203D-06+(azi(ijk)- 0.10D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.3162900000000000D-04+(azi(ijk)- 0.10D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk, 11)= 0.3647887405788254D-11
       rk(ijk, 11)=-0.5894213828945961D-08+(azi(ijk)- 0.20D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.2493726504511192D-06+(azi(ijk)- 0.20D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.2975920000000000D-04+(azi(ijk)- 0.20D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk, 11)= 0.2530332876024747D-10
       rk(ijk, 11)=-0.5784777206772324D-08+(azi(ijk)- 0.30D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.3661625608083019D-06+(azi(ijk)- 0.30D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.2667970000000000D-04+(azi(ijk)- 0.30D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk, 11)= 0.6013879755322039D-10
       rk(ijk, 11)=-0.5025677343964817D-08+(azi(ijk)- 0.40D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.4742671063156742D-06+(azi(ijk)- 0.40D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.2246490000000000D-04+(azi(ijk)- 0.40D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk, 11)= 0.1004414810268224D-09
       rk(ijk, 11)=-0.3221513417368026D-08+(azi(ijk)- 0.50D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.5567390139290039D-06+(azi(ijk)- 0.50D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.1727980000000000D-04+(azi(ijk)- 0.50D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk, 11)= 0.3107352783395266D-09
       rk(ijk, 11)=-0.2082689865642921D-09+(azi(ijk)- 0.60D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.5910368379683177D-06+(azi(ijk)- 0.60D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.1149070000000000D-04+(azi(ijk)- 0.60D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk, 11)= 0.1345128013864191D-09
       rk(ijk, 11)= 0.9113789363622316D-08+(azi(ijk)- 0.70D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.5019816341977498D-06+(azi(ijk)- 0.70D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.5870240000000000D-05+(azi(ijk)- 0.70D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk, 11)=-0.1960445509603971D-09
       rk(ijk, 11)= 0.1234209659689431D-07+(azi(ijk)- 0.78D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.3303345465136462D-06+(azi(ijk)- 0.78D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.2506540000000000D-05+(azi(ijk)- 0.78D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk, 11)= 0.2279277828261561D-08
       rk(ijk, 11)= 0.7637027373853022D-08+(azi(ijk)- 0.86D+02) * rk(ijk, 11)
       rk(ijk, 11)=-0.1705015547476910D-06+(azi(ijk)- 0.86D+02) * rk(ijk, 11)
       rk(ijk, 11)= 0.5533830000000000D-06+(azi(ijk)- 0.86D+02) * rk(ijk, 11)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk, 11)= 0.1394430000000000D-06
      endif
      rk(ijk, 11) = rk(ijk, 11) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk, 12)=-0.1292568102250474D-10
       rk(ijk, 12)=-0.1349143189774952D-08+(azi(ijk)- 0.00D+00) * rk(ijk, 12)
       rk(ijk, 12)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk, 12)
       rk(ijk, 12)= 0.6105070000000000D-05+(azi(ijk)- 0.00D+00) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk, 12)= 0.1221704306751377D-10
       rk(ijk, 12)=-0.1736913620450095D-08+(azi(ijk)- 0.10D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.3086056810225046D-07+(azi(ijk)- 0.10D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.5957230000000000D-05+(azi(ijk)- 0.10D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk, 12)= 0.7227508752451125D-11
       rk(ijk, 12)=-0.1370402328424683D-08+(azi(ijk)- 0.20D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.6193372759099823D-07+(azi(ijk)- 0.20D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.5487150000000000D-05+(azi(ijk)- 0.20D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk, 12)= 0.1521292192267961D-10
       rk(ijk, 12)=-0.1153577065851150D-08+(azi(ijk)- 0.30D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.8717352153375653D-07+(azi(ijk)- 0.30D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.4738000000000000D-05+(azi(ijk)- 0.30D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk, 12)= 0.2301080355683157D-10
       rk(ijk, 12)=-0.6971894081707524D-09+(azi(ijk)- 0.40D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.1056811862739757D-06+(azi(ijk)- 0.40D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.3766120000000000D-05+(azi(ijk)- 0.40D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk, 12)= 0.3106386384998460D-10
       rk(ijk, 12)=-0.6865301465772937D-11+(azi(ijk)- 0.50D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.1127217333703412D-06+(azi(ijk)- 0.50D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.2662600000000000D-05+(azi(ijk)- 0.50D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk, 12)= 0.4531874104323753D-10
       rk(ijk, 12)= 0.9250506140335778D-09+(azi(ijk)- 0.60D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.1035398802446616D-06+(azi(ijk)- 0.60D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.1565760000000000D-05+(azi(ijk)- 0.60D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk, 12)=-0.1303761111909802D-10
       rk(ijk, 12)= 0.2284612845330262D-08+(azi(ijk)- 0.70D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.7144324565101817D-07+(azi(ijk)- 0.70D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.6681850000000000D-06+(azi(ijk)- 0.70D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk, 12)=-0.4077238230008851D-10
       rk(ijk, 12)= 0.1971710178474831D-08+(azi(ijk)- 0.78D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.3739266146060570D-07+(azi(ijk)- 0.78D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.2361790000000000D-06+(azi(ijk)- 0.78D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk, 12)= 0.1193377495925112D-09
       rk(ijk, 12)= 0.9931730032745764D-09+(azi(ijk)- 0.86D+02) * rk(ijk, 12)
       rk(ijk, 12)=-0.1367359600664033D-07+(azi(ijk)- 0.86D+02) * rk(ijk, 12)
       rk(ijk, 12)= 0.4235170000000000D-07+(azi(ijk)- 0.86D+02) * rk(ijk, 12)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk, 12)= 0.1118570000000000D-07
      endif
      rk(ijk, 12) = rk(ijk, 12) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk, 13)=-0.2724056225766428D-11
       rk(ijk, 13)=-0.2068594377423349D-09+(azi(ijk)- 0.00D+00) * rk(ijk, 13)
       rk(ijk, 13)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk, 13)
       rk(ijk, 13)= 0.1769770000000000D-05+(azi(ijk)- 0.00D+00) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk, 13)= 0.1042168677298970D-11
       rk(ijk, 13)=-0.2885811245153277D-09+(azi(ijk)- 0.10D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.4954405622576625D-08+(azi(ijk)- 0.10D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.1746360000000000D-05+(azi(ijk)- 0.10D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk, 13)=-0.7346184834292973D-12
       rk(ijk, 13)=-0.2573160641963587D-09+(azi(ijk)- 0.20D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.1041337750969349D-07+(azi(ijk)- 0.20D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.1669000000000000D-05+(azi(ijk)- 0.20D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk, 13)=-0.4136947435816487D-12
       rk(ijk, 13)=-0.2793546186992379D-09+(azi(ijk)- 0.30D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.1578008433864945D-07+(azi(ijk)- 0.30D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.1538400000000000D-05+(azi(ijk)- 0.30D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk, 13)= 0.3593974577551793D-12
       rk(ijk, 13)=-0.2917654610066857D-09+(azi(ijk)- 0.40D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.2149128513570870D-07+(azi(ijk)- 0.40D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.1352250000000000D-05+(azi(ijk)- 0.40D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk, 13)= 0.1158104912560934D-11
       rk(ijk, 13)=-0.2809835372740307D-09+(azi(ijk)- 0.50D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.2721877511851585D-07+(azi(ijk)- 0.50D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.1108520000000000D-05+(azi(ijk)- 0.50D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk, 13)= 0.1481218289200117D-10
       rk(ijk, 13)=-0.2462403898972166D-09+(azi(ijk)- 0.60D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.3249101439022824D-07+(azi(ijk)- 0.60D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.8093920000000000D-06+(azi(ijk)- 0.60D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk, 13)= 0.2251236790109542D-10
       rk(ijk, 13)= 0.1981250968627303D-09+(azi(ijk)- 0.70D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.3297216732057239D-07+(azi(ijk)- 0.70D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.4746700000000000D-06+(azi(ijk)- 0.70D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk, 13)=-0.2282006684613484D-10
       rk(ijk, 13)= 0.7384219264892082D-09+(azi(ijk)- 0.78D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.2547979113375829D-07+(azi(ijk)- 0.78D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.2350990000000000D-06+(azi(ijk)- 0.78D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk, 13)= 0.3441785534780930D-09
       rk(ijk, 13)= 0.1907403221803731D-09+(azi(ijk)- 0.86D+02) * rk(ijk, 13)
       rk(ijk, 13)=-0.1804649314438283D-07+(azi(ijk)- 0.86D+02) * rk(ijk, 13)
       rk(ijk, 13)= 0.6683580000000001D-07+(azi(ijk)- 0.86D+02) * rk(ijk, 13)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk, 13)= 0.1972910000000000D-07
      endif
      rk(ijk, 13) = rk(ijk, 13) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk, 14)=-0.1826021023809207D-09
       rk(ijk, 14)=-0.1228397897619083D-07+(azi(ijk)- 0.00D+00) * rk(ijk, 14)
       rk(ijk, 14)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk, 14)
       rk(ijk, 14)= 0.1204770000000000D-03+(azi(ijk)- 0.00D+00) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk, 14)= 0.7680630714277246D-10
       rk(ijk, 14)=-0.1776204204761845D-07+(azi(ijk)- 0.10D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.3004602102380928D-06+(azi(ijk)- 0.10D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.1190660000000000D-03+(azi(ijk)- 0.10D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk, 14)=-0.4662312619017367D-10
       rk(ijk, 14)=-0.1545785283333529D-07+(azi(ijk)- 0.20D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.6326591590476302D-06+(azi(ijk)- 0.20D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.1143620000000000D-03+(azi(ijk)- 0.20D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk, 14)=-0.3391380238206501D-10
       rk(ijk, 14)=-0.1685654661904052D-07+(azi(ijk)- 0.30D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.9558031535713880D-06+(azi(ijk)- 0.30D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.1064430000000000D-03+(azi(ijk)- 0.30D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk, 14)=-0.3482166428159648D-10
       rk(ijk, 14)=-0.1787396069050235D-07+(azi(ijk)- 0.40D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.1303108226666817D-05+(azi(ijk)- 0.40D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.9516540000000000D-04+(azi(ijk)- 0.40D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk, 14)=-0.9419954049156243D-10
       rk(ijk, 14)=-0.1891861061895049D-07+(azi(ijk)- 0.50D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.1671033939761342D-05+(azi(ijk)- 0.50D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.8031210000000001D-04+(azi(ijk)- 0.50D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk, 14)= 0.4307198262481168D-09
       rk(ijk, 14)=-0.2174459683369922D-07+(azi(ijk)- 0.60D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.2077666014287815D-05+(azi(ijk)- 0.60D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.6161569999999999D-04+(azi(ijk)- 0.60D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk, 14)= 0.1932828429021196D-08
       rk(ijk, 14)=-0.8823002046251036D-08+(azi(ijk)- 0.70D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.2383342003087402D-05+(azi(ijk)- 0.70D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.3909530000000000D-04+(azi(ijk)- 0.70D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk, 14)= 0.7078805395488009D-10
       rk(ijk, 14)= 0.3756488025026885D-07+(azi(ijk)- 0.78D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.2153406977455393D-05+(azi(ijk)- 0.78D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.2045350000000000D-04+(azi(ijk)- 0.78D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk, 14)= 0.2551390080692380D-07
       rk(ijk, 14)= 0.3926379354514129D-07+(azi(ijk)- 0.86D+02) * rk(ijk, 14)
       rk(ijk, 14)=-0.1538777587091831D-05+(azi(ijk)- 0.86D+02) * rk(ijk, 14)
       rk(ijk, 14)= 0.5666640000000000D-05+(azi(ijk)- 0.86D+02) * rk(ijk, 14)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk, 14)= 0.1772640000000000D-05
      endif
      rk(ijk, 14) = rk(ijk, 14) * att(ijk)
!
!
      if(azi(ijk).lt.0.D0)then
       STOP
      elseif(azi(ijk).ge. 0.00D+00 .and. azi(ijk).lt. 0.10D+02) then
       rk(ijk, 15)=-0.1826021023809207D-09
       rk(ijk, 15)=-0.1228397897619083D-07+(azi(ijk)- 0.00D+00) * rk(ijk, 15)
       rk(ijk, 15)= 0.0000000000000000D+00+(azi(ijk)- 0.00D+00) * rk(ijk, 15)
       rk(ijk, 15)= 0.1204770000000000D-03+(azi(ijk)- 0.00D+00) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.10D+02 .and. azi(ijk).lt. 0.20D+02) then
       rk(ijk, 15)= 0.7680630714277246D-10
       rk(ijk, 15)=-0.1776204204761845D-07+(azi(ijk)- 0.10D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.3004602102380928D-06+(azi(ijk)- 0.10D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.1190660000000000D-03+(azi(ijk)- 0.10D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.20D+02 .and. azi(ijk).lt. 0.30D+02) then
       rk(ijk, 15)=-0.4662312619017367D-10
       rk(ijk, 15)=-0.1545785283333529D-07+(azi(ijk)- 0.20D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.6326591590476302D-06+(azi(ijk)- 0.20D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.1143620000000000D-03+(azi(ijk)- 0.20D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.30D+02 .and. azi(ijk).lt. 0.40D+02) then
       rk(ijk, 15)=-0.3391380238206501D-10
       rk(ijk, 15)=-0.1685654661904052D-07+(azi(ijk)- 0.30D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.9558031535713880D-06+(azi(ijk)- 0.30D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.1064430000000000D-03+(azi(ijk)- 0.30D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.40D+02 .and. azi(ijk).lt. 0.50D+02) then
       rk(ijk, 15)=-0.3482166428159648D-10
       rk(ijk, 15)=-0.1787396069050235D-07+(azi(ijk)- 0.40D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.1303108226666817D-05+(azi(ijk)- 0.40D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.9516540000000000D-04+(azi(ijk)- 0.40D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.50D+02 .and. azi(ijk).lt. 0.60D+02) then
       rk(ijk, 15)=-0.9419954049156243D-10
       rk(ijk, 15)=-0.1891861061895049D-07+(azi(ijk)- 0.50D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.1671033939761342D-05+(azi(ijk)- 0.50D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.8031210000000001D-04+(azi(ijk)- 0.50D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.60D+02 .and. azi(ijk).lt. 0.70D+02) then
       rk(ijk, 15)= 0.4307198262481168D-09
       rk(ijk, 15)=-0.2174459683369922D-07+(azi(ijk)- 0.60D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.2077666014287815D-05+(azi(ijk)- 0.60D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.6161569999999999D-04+(azi(ijk)- 0.60D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.70D+02 .and. azi(ijk).lt. 0.78D+02) then
       rk(ijk, 15)= 0.1932828429021196D-08
       rk(ijk, 15)=-0.8823002046251036D-08+(azi(ijk)- 0.70D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.2383342003087402D-05+(azi(ijk)- 0.70D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.3909530000000000D-04+(azi(ijk)- 0.70D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.78D+02 .and. azi(ijk).lt. 0.86D+02) then
       rk(ijk, 15)= 0.7078805395488009D-10
       rk(ijk, 15)= 0.3756488025026885D-07+(azi(ijk)- 0.78D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.2153406977455393D-05+(azi(ijk)- 0.78D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.2045350000000000D-04+(azi(ijk)- 0.78D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge. 0.86D+02 .and. azi(ijk).lt. 0.90D+02) then
       rk(ijk, 15)= 0.2551390080692380D-07
       rk(ijk, 15)= 0.3926379354514129D-07+(azi(ijk)- 0.86D+02) * rk(ijk, 15)
       rk(ijk, 15)=-0.1538777587091831D-05+(azi(ijk)- 0.86D+02) * rk(ijk, 15)
       rk(ijk, 15)= 0.5666640000000000D-05+(azi(ijk)- 0.86D+02) * rk(ijk, 15)
      elseif(azi(ijk).ge.90.D0)then
       rk(ijk, 15)= 0.1772640000000000D-05
      endif
      rk(ijk, 15) = rk(ijk, 15) * att(ijk)
!
      rk(ijk, 16) = SumM(ijk) * 6.0d-34 * (temp(ijk)/3.d2) ** (-2.4d0)
      rk(ijk, 16) = rk(ijk, 16) * SumM(ijk) * 0.2d0
      rk(ijk, 17) =  DEXP(-0.2555157957424871D+02   &
         - (  0.2060000000000000D+04 )/temp(ijk))
      rk(ijk, 18) =  DEXP(-0.2456296818079493D+02   &
         - ( -0.1100000000000000D+03 )/temp(ijk))
      rk(ijk, 18) = rk(ijk, 18) * SumM(ijk) * 0.8d0
      rk(ijk, 19) =  DEXP(-0.2413451355446207D+02   &
         - ( -0.5500000000000000D+02 )/temp(ijk))
      rk(ijk, 19) = rk(ijk, 19) * SumM(ijk) * 0.2d0
      rk(ijk, 20) =  DEXP(-0.2253727091512179D+02   &
         - ( -0.6000000000000000D+02 )/temp(ijk))
      rk(ijk, 20) = rk(ijk, 20) * YlH2O(ijk)
      rk(ijk, 21) =  DEXP(-0.2710039286486638D+02   &
         - (  0.9400000000000000D+03 )/temp(ijk))
      rk(ijk, 22) =  DEXP(-0.3214088112211231D+02   &
         - (  0.4900000000000000D+03 )/temp(ijk))
      rk(ijk, 23) =  DEXP(-0.2375982010502066D+02   &
         - ( -0.2500000000000000D+03 )/temp(ijk))
      rk(ijk, 24) =  0.1800000000000000D-11
      rk(ijk, 25) =  DEXP(-0.2868084324042723D+02   &
         - ( -0.4300000000000000D+03 )/temp(ijk))
      rk(ijk, 26) = 1.7d-33 * SumM(ijk) *    &
                    dexp(1000.0d0 / temp(ijk)) * (1 + 1.4D-21 *    &
                    YlH2O(ijk) * dexp(2200.0d0 / temp(ijk)))
      Effko(ijk) =  0.9000000000000000D-31* (temp(ijk) / 3.d2)   &
                   **(- ( 0.1500000000000000D+01))
      Rapk(ijk) =  0.3000000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.0000000000000000D+00))
      rk(ijk, 27) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 28) =  DEXP(-0.2600178057619827D+02   &
         - ( -0.2100000000000000D+03 )/temp(ijk))
      Effko(ijk) =  0.2500000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.1800000000000000D+01))
      Rapk(ijk) =  0.2200000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.7000000000000000D+00))
      rk(ijk, 29) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      Effko(ijk) =  0.7000000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.2600000000000000D+01))
      Rapk(ijk) =  0.3600000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1000000000000000D+00))
      rk(ijk, 30) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      Effko(ijk) =  0.1800000000000000D-29* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3000000000000000D+01))
      Rapk(ijk) =  0.2800000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.0000000000000000D+00))
      rk(ijk, 31) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 32) =  0.2200000000000000D-10
      rk(ijk, 33) =  DEXP(-0.2637825814743318D+02   &
         - ( -0.2500000000000000D+03 )/temp(ijk))
      Effko(ijk) =  0.2000000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3400000000000000D+01))
      Rapk(ijk) =  0.2900000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1100000000000000D+01))
      rk(ijk, 34) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 35) =  DEXP( 0.6142746008608852D+02   &
         - (  0.1090000000000000D+05 )/temp(ijk))
      Effko(ijk) =  0.2000000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3400000000000000D+01))
      Rapk(ijk) =  0.2900000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1100000000000000D+01))
      facteur(ijk) = (Effko(ijk) * SumM(ijk) / ( 1.0d0 + Effko(ijk) * SumM(ijk) /    &
                 Rapk(ijk))) * 0.6d0 ** (1.0d0 / (1.0d0 +    &
                  (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 35) = facteur(ijk) * rk(ijk, 35)
      rk(ijk, 36) =  0.3500000000000000D-11
      rk(ijk, 37) =  DEXP(-0.2474064935803238D+02   &
         - (  0.3900000000000000D+03 )/temp(ijk))
      Effko(ijk) = 2.4d-14 * dexp(460.0d0 / temp(ijk))
      Rapk(ijk) = 2.7d-17 * dexp(2199.0d0 / temp(ijk))
      facteur(ijk) = 6.5d-34 * dexp(1335.0d0 / temp(ijk)) * SumM(ijk)
      rk(ijk, 38) = Effko(ijk) + facteur(ijk)/(1.0d0 + facteur(ijk) / Rapk(ijk))
      rk(ijk, 39) =  DEXP(-0.2736865685146106D+02   &
         - ( -0.3800000000000000D+03 )/temp(ijk))
      rk(ijk, 40) =  DEXP(-0.2653240882726044D+02   &
         - (  0.1500000000000000D+04 )/temp(ijk))
      rk(ijk, 41) =  DEXP(-0.2975128465212864D+02   &
         - (  0.2450000000000000D+04 )/temp(ijk))
      rk(ijk, 42) =  DEXP(-0.8860689615829534D+02   &
         - ( -0.5300000000000000D+03 )/temp(ijk))
      rk(ijk, 42) = rk(ijk, 42) * SumM(ijk) * 0.2d0
      rk(ijk, 43) =  0.4400000000000000D-39
      rk(ijk, 43) = rk(ijk, 43) * YlH2O(ijk)
      rk(ijk, 44) =  0.9999999999999999D-20
      rk(ijk, 45) =  DEXP(-0.2492297091482634D+02   &
         - ( -0.1700000000000000D+03 )/temp(ijk))
      rk(ijk, 46) =  DEXP(-0.3073211390514037D+02   &
         - (  0.1260000000000000D+04 )/temp(ijk))
      Effko(ijk) =  0.2000000000000000D-29* (temp(ijk) / 3.d2)   &
                   **(- ( 0.4400000000000000D+01))
      Rapk(ijk) =  0.1400000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.7000000000000000D+00))
      rk(ijk, 47) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 48) =  DEXP( 0.6117554523749536D+02   &
         - (  0.1100000000000000D+05 )/temp(ijk))
      Effko(ijk) =  0.2000000000000000D-29* (temp(ijk) / 3.d2)   &
                   **(- ( 0.4400000000000000D+01))
      Rapk(ijk) =  0.1400000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.7000000000000000D+00))
      facteur(ijk) = (Effko(ijk) * SumM(ijk) / ( 1.0d0 + Effko(ijk) * SumM(ijk) /    &
                 Rapk(ijk))) * 0.6d0 ** (1.0d0 / (1.0d0 +    &
                  (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 48) = facteur(ijk) * rk(ijk, 48)
      rk(ijk, 49) =  0.1500000000000000D-20
      rk(ijk, 49) = rk(ijk, 49) * YlH2O(ijk)
      rk(ijk, 50) = 1.80d-39 * YlH2O(ijk) * YlH2O(ijk)
      rk(ijk, 51) =  DEXP(-0.2779354004542632D+02   &
         - (  0.2450000000000000D+04 )/temp(ijk))
      Effko(ijk) =  0.5900000000000000D-32* (temp(ijk) / 3.d2)   &
                   **(- ( 0.1400000000000000D+01))
      Rapk(ijk) =  0.1100000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- (-0.1300000000000000D+01))
      rk(ijk, 52) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      Effko(ijk) = 1.5d-13  * (temp(ijk)/300.D0)**(0.6D0)
      Rapk(ijk) = 2.1d+09 * (temp(ijk)/300.D0)**(6.1D0)
      rk(ijk, 53)=(Effko(ijk)/(1.D0+Effko(ijk)/(Rapk(ijk)/SumM(ijk)))) *    &
        0.6D0**(1.0d0+(DLOG10(Effko(ijk)/(Rapk(ijk)/SumM(ijk)))**2.0D0)**(-1.0D0))
      rk(ijk, 54) =  DEXP(-0.2410466059131239D+02   &
         - (  0.1600000000000000D+04 )/temp(ijk))
      rk(ijk, 55) =  DEXP(-0.2592627302369012D+02   &
         - ( -0.1250000000000000D+03 )/temp(ijk))
      rk(ijk, 56) =  0.5800000000000000D-15
      rk(ijk, 57) =  DEXP(-0.2474064935803238D+02   &
         - (  0.1100000000000000D+04 )/temp(ijk))
      rk(ijk, 58) =  DEXP(-0.2614941657500433D+02   &
         - ( -0.3650000000000000D+03 )/temp(ijk))
      rk(ijk, 59) =  DEXP(-0.2729454887930734D+02   &
         - (  0.1900000000000000D+04 )/temp(ijk))
      rk(ijk, 60) =  DEXP(-0.2561611809538628D+02   &
         - ( -0.2900000000000000D+03 )/temp(ijk))
      Effko(ijk) =  0.9700000000000000D-28* (temp(ijk) / 3.d2)   &
                   **(- ( 0.5600000000000000D+01))
      Rapk(ijk) =  0.9300000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1500000000000000D+01))
      rk(ijk, 61) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 62) =  DEXP( 0.6457674261915753D+02   &
         - (  0.1400000000000000D+05 )/temp(ijk))
      Effko(ijk) =  0.9700000000000000D-28* (temp(ijk) / 3.d2)   &
                   **(- ( 0.5600000000000000D+01))
      Rapk(ijk) =  0.9300000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1500000000000000D+01))
      facteur(ijk) = (Effko(ijk) * SumM(ijk) / ( 1.0d0 + Effko(ijk) * SumM(ijk) /    &
                 Rapk(ijk))) * 0.6d0 ** (1.0d0 / (1.0d0 +    &
                  (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 62) = facteur(ijk) * rk(ijk, 62)
      rk(ijk, 63) =  DEXP(-0.2847499118622308D+02   &
         - ( -0.1040000000000000D+04 )/temp(ijk))
      rk(ijk, 64) =  DEXP(-0.2660140169874739D+02   &
         - ( -0.5300000000000000D+03 )/temp(ijk))
      rk(ijk, 65) =  0.8140000000000000D-12
      rk(ijk, 66) =  DEXP(-0.1009950243646209D+02   &
         - (  0.8000000000000000D+04 )/temp(ijk))
      rk(ijk, 66) = rk(ijk, 66) * SumM(ijk)
      rk(ijk, 67) =  0.3290000000000000D-15
      rk(ijk, 67) = rk(ijk, 67) * SumM(ijk) * 0.2d0
      rk(ijk, 68) =  0.1500000000000000D-10
      rk(ijk, 69) =  DEXP(-0.2528921530978122D+02   &
         - (  0.7920000000000000D+03 )/temp(ijk))
      Effko(ijk) =  0.1000000000000000D-27* (temp(ijk) / 3.d2)   &
                   **(- ( 0.4500000000000000D+01))
      Rapk(ijk) =  0.8800000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.8500000000000000D+00))
      rk(ijk, 70) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 71) =  DEXP(-0.3205386974512268D+02   &
         - (  0.2630000000000000D+04 )/temp(ijk))
      rk(ijk, 72) =  DEXP(-0.2409006179189123D+02   &
         - (  0.3400000000000000D+03 )/temp(ijk))
      rk(ijk, 73) =  DEXP(-0.2544834631960706D+02   &
         - ( -0.4780000000000000D+03 )/temp(ijk))
      rk(ijk, 74) =  DEXP(-0.3339137394207314D+02   &
         - (  0.1680000000000000D+04 )/temp(ijk))
      rk(ijk, 75) =  DEXP(-0.2937972109569615D+02   &
         - (  0.1500000000000000D+02 )/temp(ijk))
      rk(ijk, 76) =  0.3600000000000000D-10
      rk(ijk, 77) =  DEXP(-0.2439627194190406D+02   &
         - ( -0.4076000000000000D+03 )/temp(ijk))
      rk(ijk, 78) =  DEXP(-0.3247698978846957D+02   &
         - (  0.1912000000000000D+04 )/temp(ijk))
      rk(ijk, 79) =  DEXP(-0.2652245849640727D+02   &
         - (  0.4480000000000000D+03 )/temp(ijk))
      rk(ijk, 80) =  0.1500000000000000D-18
      rk(ijk, 81) =  0.3360000000000000D-10
      rk(ijk, 82) =  0.7110000000000000D-17
      rk(ijk, 83) =  0.1000000000000000D-14
      rk(ijk, 84) =  DEXP(-0.2704323445102643D+02   &
         - ( -0.3550000000000000D+03 )/temp(ijk))
      rk(ijk, 85) =  0.8100000000000000D-11
      rk(ijk, 86) =  0.1720000000000000D-18
      rk(ijk, 86) = rk(ijk, 86) * SumM(ijk)
      rk(ijk, 87) =  0.4100000000000000D-10
      rk(ijk, 88) =  0.2200000000000000D-10
      rk(ijk, 89) =  0.1400000000000000D-10
      rk(ijk, 90) =  0.3000000000000000D-10
      rk(ijk, 91) =  DEXP(-0.3745754762732854D+02   &
         - (  0.5000000000000000D+03 )/temp(ijk))
      rk(ijk, 92) =  DEXP(-0.2479780777187233D+02   &
         - ( -0.1160000000000000D+03 )/temp(ijk))
      rk(ijk, 93) =  0.1700000000000000D-10
      rk(ijk, 94) =  DEXP(-0.2653240882726044D+02   &
         - ( -0.2800000000000000D+03 )/temp(ijk))
      rk(ijk, 95) =  DEXP(-0.2653240882726044D+02   &
         - ( -0.2800000000000000D+03 )/temp(ijk))
      rk(ijk, 96) =  DEXP(-0.2901731547704844D+02   &
         - ( -0.1900000000000000D+03 )/temp(ijk))
      rk(ijk, 97) =  DEXP(-0.2859860514219026D+02   &
         - ( -0.8000000000000000D+03 )/temp(ijk))
      rk(ijk, 98) =  DEXP(-0.2859860514219026D+02   &
         - ( -0.8000000000000000D+03 )/temp(ijk))
      rk(ijk, 99) =  DEXP(-0.2901731547704844D+02   &
         - ( -0.1900000000000000D+03 )/temp(ijk))
      rk(ijk,100) =  DEXP(-0.2832416829648849D+02   &
         - ( -0.1900000000000000D+03 )/temp(ijk))
      rk(ijk,101) =  DEXP(-0.2673493309137191D+02   &
         - (  0.1775000000000000D+04 )/temp(ijk))
      rk(ijk,102) =  DEXP(-0.2656631037893612D+02   &
         - (  0.3450000000000000D+03 )/temp(ijk))
      rk(ijk,103) =  DEXP(-0.2653240882726044D+02   &
         - (  0.2000000000000000D+02 )/temp(ijk))
       END DO
 
   END SUBROUTINE kinetic
 
  END MODULE mod_chem_spack_kinetic
 
